import os
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Button
from matplotlib.widgets import Slider
import h5py

from stimugenerator import visualization
from stimugenerator import dataIO


def getDistance(cropName):
    f = h5py.File(cropName,'r')
    distance = int(f['/distance'][0])
    f.close()
    
    return distance

def getCrop(cropName):
    f = h5py.File(cropName,'r')
    crop = np.array(f['/crop'])
    f.close()
    
    return crop

def setTitle(cropList,imgIndx):

    cropName = cropList[imgIndx].split('/')[-1].strip('_1.hdf5')
    distance = getDistance(cropList[imgIndx])
            
    title = cropName+'_'+str(distance)
    return title

videoPath = "../data/mouse_footage/"
videoName = "20180713_13_1.h5"

movie = dataIO.read_movie_from_h5(videoPath+videoName)

# declaration
global i

# manuelly reset everytime the program crashes -> find out why it crashes/gets slower
reStartIndex = 0
i = reStartIndex

plt.rcParams["figure.figsize"] = [7.50, 7.50]
plt.rcParams["figure.autolayout"] = True

fig, ax = plt.subplots()
# show the i-th frame of the loaded video as the starting frame
img = ax.imshow(visualization.img_real2view(movie[i][...,::-1]))

axes1 = plt.axes([0.41, 0.000001, 0.21, 0.075])
bnext1 = Button(axes1, 'next frame',color="green")

axes2 = plt.axes([0.61, 0.000001, 0.21, 0.075])
bnext2 = Button(axes2, 'sH_w_Obstacles',color="yellow")

axes3 = plt.axes([0.81, 0.000001, 0.21, 0.075])
bnext3 = Button(axes3, 'next',color="red")

axes4 = plt.axes([0.000001, 0.000001, 0.21, 0.075])
bback = Button(axes4, 'back',color="Red")

axes5 = plt.axes([0.000001, 0.91, 0.21, 0.075])
bdelete = Button(axes5, 'trash',color="purple")
# button function
def showNextFrame(val):
    global i 
    i = i+1
   
    ax.imshow(visualization.img_real2view(movie[i][...,::-1]))
    ax.set_title('Frame: '+str(i))
    fig.canvas.draw_idle()

def straightH_withObstacles(val):
    global i 

    # save
    with open('sH_w_Obstacles.txt','a+') as f:
        f.write(horizonCrops[i])
        f.write('\n')
    f.close()
    print("saved: ",horizonCrops[i])

    # show next
    i = i + 1
    print("show next no.:",i)
    
    ax.imshow(visualization.img_real2view(movie[i][...,::-1]))
    title = setTitle(horizonCrops,i)
    ax.set_title(title)

    fig.canvas.draw_idle()

def update(val):
    global i 
    i = i + 1
    ax.imshow(visualization.img_real2view(movie[i][...,::-1]))
    ax.set_title('Frame: '+str(i))
    fig.canvas.draw_idle()
    print(i)

def goback(val):
    global i 
    if i>=1:
        i = i - 1
    else:
        i = reStartIndex
    ax.imshow(visualization.img_real2view(movie[i][...,::-1]))
    ax.set_title('Frame: '+str(i))
    fig.canvas.draw_idle()
    print("show previous no.:",i)

def delete(val):
    global i 
    # save
    with open('absoluteTrashCrops.txt','a+') as f:
        f.write(horizonCrops[i])
        f.write('\n')
    f.close()
    print("saved: ",horizonCrops[i])

    # show next
    i = i + 1
    print("show next no.:",i)
    ax.imshow(visualization.img_real2view(movie[i][...,::-1]))

    ax.set_title('Frame: '+str(i))

    fig.canvas.draw_idle()
    

bnext1.on_clicked(showNextFrame) # show next frame 
bnext2.on_clicked(straightH_withObstacles)
bnext3.on_clicked(update)

bback.on_clicked(goback)
bdelete.on_clicked(delete)
plt.show()



import numpy as np
# to better visulaize image, use gamma correction to transfer image real to image view
def img_real2view(img):
    gamma_correction=lambda x:np.power(x,1.0/2.2)
    img_shape=img.shape
    # gray image
    if np.size(img_shape)==2:
        # uint8
        if np.max(img)>1:
            temp_view=np.zeros_like(img,dtype=np.float32)
            temp_view=np.float32(img)/255.0 # float32, 1.0
            temp_view=gamma_correction(temp_view)
            temp_view2=np.zeros_like(img,dtype=np.uint8)
            temp_view2=np.uint8(temp_view*255)
            return temp_view2
        # float
        if np.max(img)<2:
            return gamma_correction(img)
            
    # color image
    if np.size(img_shape)==3:
        # uint8, BGR
        if np.max(img)>1:
            temp_view=np.zeros_like(img,dtype=np.float32)
            temp_view=np.float32(img[...,::-1])/255.0 # gb,1.0
            temp_view[...,-1]=gamma_correction(temp_view[...,-1])
            temp_view[...,1]=gamma_correction(temp_view[...,1])
            temp_view2=np.zeros_like(img,dtype=np.uint8)
            temp_view2=np.uint8(temp_view[...,::-1]*255) # bgr,255
            return temp_view2
        # float, RGB
        if np.max(img)<2:
            return gamma_correction(img)

def read_movie_from_h5(filename):
    h5f = h5py.File(filename,'r')
    movie_bgr_h5=h5f['movie_bgr_real'][:]
    h5f.close()
    return movie_bgr_h5



import os
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Button
import h5py
import cv2 as cv
import random

from stimugenerator import visualization
from stimugenerator import dataIO

videoPath = "../data/mouse_footage/"
videoName = "20180713_13_1.h5"
movie = dataIO.read_movie_from_h5(videoPath+videoName)
lenMovie = len(movie)

# button section
bt_bottom_pos = 0.002
bt_width = 0.1
bt_height = 0.025

# (fast) for/back-ward stepsize
smallStep = 1
bigStep = 10
# declaration
global frameNr
global saveClip
global crop_x
global crop_y
# manuelly reset everytime the program crashes -> find out why it crashes/gets slower
reStartIndex = 0
frameNr = reStartIndex

plt.rcParams["figure.figsize"] = [7.50, 7.50]
plt.rcParams["figure.autolayout"] = True
plt.rcParams["xtick.labelsize"] = 8
plt.rcParams["ytick.labelsize"] = 8


fig = plt.figure(constrained_layout=True, figsize=(16, 8))
subfigs = fig.subfigures(1,2)

# the left panel
axLeft = subfigs[0].subplots()
axLeft.imshow(visualization.img_real2view(movie[frameNr][...,::-1]))
axLeft.set_title("Frame "+str(frameNr)+"/"+str(lenMovie),fontsize = 10)

axRight = subfigs[1].subplots(3,5)
axRight = axRight.flatten()
for each in axRight:
    each.axis("off")

fastBackwardAx = subfigs[0].add_axes([0.1, bt_bottom_pos, bt_width, bt_height])
fbw_bt = Button(fastBackwardAx, color = "red",label='-10')

backwardAx = subfigs[0].add_axes([0.25, bt_bottom_pos, bt_width, bt_height])
bw_bt = Button(backwardAx, color = "red",label='-1')

forwardAx = subfigs[0].add_axes([0.6, bt_bottom_pos, bt_width, bt_height])
fw_bt = Button(forwardAx, color = "green",label='+1')

fastForwardAx = subfigs[0].add_axes([0.75, bt_bottom_pos, bt_width, bt_height])
ffw_bt = Button(fastForwardAx, color = "green",label='+10')

def prevOne(val):
    global frameNr
    axLeft.clear()

    if frameNr>=smallStep:
        frameNr = frameNr-smallStep
    axLeft.imshow(visualization.img_real2view(movie[frameNr][...,::-1]))
    axLeft.set_title("Frame "+str(frameNr)+"/"+str(lenMovie),fontsize = 10)
    subfigs[0].canvas.draw_idle()

def nextOne(val):
    global frameNr
    axLeft.clear()

    frameNr = frameNr+smallStep

    axLeft.imshow(visualization.img_real2view(movie[frameNr][...,::-1]))
    axLeft.set_title("Frame "+str(frameNr)+"/"+str(lenMovie),fontsize = 10)
    subfigs[0].canvas.draw_idle()

def prevMore(val):
    global frameNr
    axLeft.clear()

    if frameNr>=bigStep:
        frameNr = frameNr-bigStep
    axLeft.imshow(visualization.img_real2view(movie[frameNr][...,::-1]))
    axLeft.set_title("Frame "+str(frameNr)+"/"+str(lenMovie),fontsize = 10)
    subfigs[0].canvas.draw_idle()

def nextMore(val):
    global frameNr
    axLeft.clear()

    frameNr = frameNr+bigStep

    axLeft.imshow(visualization.img_real2view(movie[frameNr][...,::-1]))
    axLeft.set_title("Frame "+str(frameNr)+"/"+str(lenMovie),fontsize = 10)
    subfigs[0].canvas.draw_idle()

fbw_bt.on_clicked(prevMore)
bw_bt.on_clicked(prevOne)

fw_bt.on_clicked(nextOne)
ffw_bt.on_clicked(nextMore)

# event handling on the left figures
# upon selection a crop on one frame
def onclick(event):
    global frameNr
    global saveClip
    global crop_x
    global crop_y
 
    click_x,click_y = round(event.xdata),round(event.ydata)
    if click_x+click_y<=2:
        print("not a valid crop")
    else:
        crop_x = click_x
        crop_y = click_y
        axLeft.clear()
        for each in axRight: 
            each.clear()
        print("x-coor: "+str(click_x)+", y-coor: "+str(click_y))

        # on the left panel 
        toDraw = np.copy(movie[frameNr])

        # on the right panel
        toPlot = np.copy(movie[frameNr:frameNr+50,click_y:click_y+18,click_x:click_x+16,:]) 
        
        # prepare the 50-frame clip to save
        saveClip = np.zeros((1,2,50,18,16))
        saveClip[0,0,:,:,:] = np.copy(toPlot[:,:,:,0]);saveClip[0,1,:,:,:] = np.copy(toPlot[:,:,:,1])
        # print(saveClip.shape)
        # print(toPlot.shape)# shape 50 18 16 3
        # print(toDraw.shape)

        # visualize the 18x16 crop
        cv.rectangle(toDraw,(click_x,click_y),(click_x+16,click_y+18),color = (255,255,255),thickness=1) # shows the starting position of the crop
        axLeft.imshow(visualization.img_real2view(toDraw[...,::-1]))
        axLeft.set_title("Frame "+str(frameNr)+"/"+str(lenMovie),fontsize = 10)

        # randomly select 5 frame 
        randomIdx = sorted(random.sample(range(0, 50), 5))
        for i in range(5):
            axRight[i].imshow(visualization.img_real2view(toPlot[randomIdx[i]][...,::-1]))
            axRight[i+5].imshow(saveClip[0,0,randomIdx[i],:,:]) # UV sanity check
            axRight[i+10].imshow(saveClip[0,1,randomIdx[i],:,:]) # Green sanity check
            axRight[i].set_title("frame: "+str(randomIdx[i]))
            axRight[i].axis("off")
        subfigs[1].canvas.draw_idle()

leftPanel_cid = subfigs[0].canvas.mpl_connect('button_press_event', onclick)

# save button
saveAx = subfigs[0].add_axes([0.4, bt_bottom_pos, bt_width, bt_height])
save_bt = Button(saveAx, color = "blue",label='save clip')

def saveClip(val):
    global frameNr
    global saveClip
    global crop_x
    global crop_y
    print("save a clip")
    saveFileName = videoName.split(".")[0]+"_"+str(frameNr)+"_"+str(crop_x)+"_"+str(crop_y)+'.npy'
    #print(saveFileName)
    if np.sum(saveClip)>0:
        np.save('../data/stimuli/sky/'+saveFileName,saveClip)

save_bt.on_clicked(saveClip)

plt.show()

